/*
 * exe.c
 *
 * Created: 2016-05-13 14:26:42
 *  Author: dsosnowski
 */ 

#define F_CPU 11059200UL
#include <avr/io.h>
#include <avr/interrupt.h>
#include <inttypes.h>
#include <util/delay.h>
#include "defs.h"
#include "comm.h"
#include "exe.h"
#include "settings.h"


//#define EXE_NUM_OF_OUTPUTS			2
//#define EXE_NUM_OF_INPUTS			4

#define STATE_ACTIVE				1
#define STATE_IDLE					0
#define STATE_ACTION				1

#define INPUT_THRESHOLD_CYCLE		3


const char output_mask[EXE_NUM_OF_OUTPUTS] = {(1<<OUT1_PIN), (1<<OUT2_PIN)};
const char input_mask[EXE_NUM_OF_INPUTS] = {(1<<ALARM1_PIN), (1<<ALARM2_PIN), (1<<SWITCH1_PIN), (1<<SWITCH2_PIN)};


struct {
	volatile char tick;
	char state[EXE_NUM_OF_OUTPUTS];
	char timestate[EXE_NUM_OF_OUTPUTS];
	uint16_t time[EXE_NUM_OF_OUTPUTS];
	char inputs[EXE_NUM_OF_INPUTS];
	char action;
} exe;


//-----------------------------------------------------
void ExeInit(void) {
	FACTORY_PORT |= (1<<FACTORY_RESET_PIN);
	
	OUT_DDR |= (1<<OUT1_PIN | 1<<OUT2_PIN);
	INPUT_DDR &= ~(1<<SWITCH1_PIN | 1<<SWITCH2_PIN | 1<<ALARM1_PIN | 1<<ALARM2_PIN);
	INPUT_PORT |= (1<<SWITCH1_PIN | 1<<SWITCH2_PIN | 1<<ALARM1_PIN | 1<<ALARM2_PIN);
	
	OCR1A = 10800;
	TCCR1A = (1<<WGM11 | 1<<WGM10);
	TCCR1B = (1<<WGM13 | 1<<WGM12 | 1<<CS12 | 1<<CS10);
	TIMSK1 = (1<<TOIE1);
}

//-----------------------------------------------------
ISR (TIMER1_OVF_vect) {
	exe.tick++;
}

//-----------------------------------------------------
void ExeCheckFactory(void) {
	if (FACTORY_RESET_GET == 0)	{
		_delay_ms(100);
		if (FACTORY_RESET_GET == 0)	{
			SettPassNew(0);
		}
	}
}

//-----------------------------------------------------
void ExeProc(void) {
	char channel;
	static char inputs[EXE_NUM_OF_INPUTS];

	for(channel=0; channel<EXE_NUM_OF_INPUTS; channel++){
		if ((INPUT_PIN & (input_mask[channel])) == 0){
			if  (inputs[channel] < INPUT_THRESHOLD_CYCLE) {
				inputs[channel]++; 
			} else {
				inputs[channel] = (2*INPUT_THRESHOLD_CYCLE);
			}
		} else {
			if  (inputs[channel] > INPUT_THRESHOLD_CYCLE) {
				inputs[channel]--;
			} else {
				inputs[channel] = 0;
			}
		}
		
		if  (inputs[channel] > INPUT_THRESHOLD_CYCLE) {
			if (exe.inputs[channel] != STATE_ACTIVE) {
				exe.inputs[channel] = STATE_ACTIVE;
				exe.action |= (STATE_ACTION << channel);
			}			
		} else {
			if (exe.inputs[channel] != STATE_IDLE) {
				exe.inputs[channel] = STATE_IDLE;
				//exe.action |= (STATE_ACTION << channel);
			}			
		}
	}

	if (exe.tick) {
		exe.tick = 0;
		for(channel=0; channel<EXE_NUM_OF_OUTPUTS; channel++){
			if (exe.time[channel] > 0) {
				exe.time[channel]--;
				if (exe.time[channel] == 0) {		
					exe.state[channel] = exe.timestate[channel];
				}
			}
		}
	}

	for(channel=0; channel<EXE_NUM_OF_OUTPUTS; channel++){
		if (exe.state[channel] >= STATE_ACTIVE) {
			OUT_PORT |= (output_mask[channel]);
		} else {
			OUT_PORT &= ~(output_mask[channel]);
		}
	}
}

//-----------------------------------------------------
char ExeActionGet(void) {
	return exe.action;
}

//-----------------------------------------------------
void ExeActionClear(void) {
	exe.action = 0;
}

//-----------------------------------------------------
char ExeInputGet(char channel) {
	if (channel < EXE_NUM_OF_INPUTS) {
		return exe.inputs[channel];	
	}
	return 0;
}

//-----------------------------------------------------
char ExeOutputGet(char channel) {
	return exe.state[channel];
}

//-----------------------------------------------------
void ExeOutputSet(char channel) {
	if (channel < EXE_NUM_OF_OUTPUTS) {
		exe.state[channel] = STATE_ACTIVE;
	}
}

//-----------------------------------------------------
void ExeOutputOff(char channel) {
	if (channel < EXE_NUM_OF_OUTPUTS) {
		exe.state[channel] = STATE_IDLE;
	}
}

//-----------------------------------------------------
void ExeOutputToggle(char channel) {
	if (channel < EXE_NUM_OF_OUTPUTS) {
		if (exe.state[channel] == STATE_IDLE) {
			exe.state[channel] = STATE_ACTIVE;		
		} else {
			exe.state[channel] = STATE_IDLE;
		}
	}
}

//-----------------------------------------------------
void ExeDelaySet(char channel, uint16_t delay) {
	if (channel < EXE_NUM_OF_OUTPUTS) {
		exe.time[channel] = delay;
				
		if (exe.state[channel] == 0) {
			exe.timestate[channel] = 1;
		} else {
			exe.timestate[channel] = 0;		
		}
	}
}

//-----------------------------------------------------
void ExeDelayStateSet(char channel, char state) {
	if (channel < EXE_NUM_OF_OUTPUTS) {
		exe.timestate[channel] = state;
	}
}

//-----------------------------------------------------
uint16_t ExeDelayGet(char channel) {
	return exe.time[channel];
}

//-----------------------------------------------------
char ExeDelayStateGet(char channel) {
	return exe.timestate[channel];
}

//-----------------------------------------------------
char ExeDelayCheck(char channel) {
	if (exe.time[channel] > 0) return 1;
	return 0;
}
